# from communities.algorithms import spectral_clustering
#
#
# class Communities:
#     def __init__(self):
#         pass
#
#     def clean(self):
#         pass
#
#     def clustering(self):
#         pass
