DEFAULT_CONFIG = """
[DEFAULT]
[PARAMETERS]
Resolution = 0.01
Count = 10
MaxRatio = 0.1
Adjacent = 3
Snr = 3.0
Threshold = 0.2
Lock = false


[METABOLITE]
Online = False
Include = config/include.csv
Exclude = config/exclude.csv
"""